@@ -1,5 +1,5 @@
 //
-// Copyright 2019 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -8,14 +8,14 @@
 // found online at https://opensource.org/licenses/MIT.
 //
 
-#ifndef NNG_H
-#define NNG_H
+#ifndef NNG_NNG_H
+#define NNG_NNG_H
 
 // NNG (nanomsg-next-gen) is an improved implementation of the SP protocols.
 // The APIs have changed, and there is no attempt to provide API compatibility
 // with legacy libnanomsg. This file defines the library consumer-facing
 // Public API. Use of definitions or declarations not found in this header
-// file is specfically unsupported and strongly discouraged.
+// file is specifically unsupported and strongly discouraged.
 
 #ifdef __cplusplus
 extern "C" {
@@ -47,10 +47,12 @@ extern "C" {
 
 // NNG Library & API version.
 // We use SemVer, and these versions are about the API, and
-// may not necessarily match the ABI versions.
+// may not necessarily match the ABI versions. Right now at
+// version 0, you should not be making any forward compatibility
+// assumptions.
 #define NNG_MAJOR_VERSION 1
-#define NNG_MINOR_VERSION 2
-#define NNG_PATCH_VERSION 4
+#define NNG_MINOR_VERSION 3
+#define NNG_PATCH_VERSION 0
 #define NNG_RELEASE_SUFFIX "" // if non-empty, this is a pre-release
 
 // Maximum length of a socket address. This includes the terminating NUL.
@@ -60,6 +62,11 @@ extern "C" {
 // with other implementations.
 #define NNG_MAXADDRLEN (128)
 
+// NNG_PROTOCOL_NUMBER is used by protocol headers to calculate their
+// protocol number from a major and minor number.  Applications should
+// probably not need to use this.
+#define NNG_PROTOCOL_NUMBER(maj, min) (((x) *16) + (y))
+
 // Types common to nng.
 
 // Identifiers are wrapped in a structure to improve compiler validation
@@ -485,7 +492,7 @@ NNG_DECL int nng_ctx_open(nng_ctx *, nng_socket);
 NNG_DECL int nng_ctx_close(nng_ctx);
 
 // nng_ctx_id returns the numeric id for the context; this will be
-// a postive value for a valid context, or < 0 for an invalid context.
+// a positive value for a valid context, or < 0 for an invalid context.
 // A valid context is not necessarily an *open* context.
 NNG_DECL int nng_ctx_id(nng_ctx);
 
@@ -676,51 +683,54 @@ NNG_DECL void nng_aio_defer(nng_aio *, nng_aio_cancelfn, void *);
 NNG_DECL void nng_sleep_aio(nng_duration, nng_aio *);
 
 // Message API.
-NNG_DECL int   nng_msg_alloc(nng_msg **, size_t);
-NNG_DECL void  nng_msg_free(nng_msg *);
-NNG_DECL int   nng_msg_realloc(nng_msg *, size_t);
-NNG_DECL void *nng_msg_header(nng_msg *);
-NNG_DECL size_t nng_msg_header_len(const nng_msg *);
-NNG_DECL void * nng_msg_body(nng_msg *);
-NNG_DECL size_t nng_msg_len(const nng_msg *);
-NNG_DECL int    nng_msg_append(nng_msg *, const void *, size_t);
-NNG_DECL int    nng_msg_insert(nng_msg *, const void *, size_t);
-NNG_DECL int    nng_msg_trim(nng_msg *, size_t);
-NNG_DECL int    nng_msg_chop(nng_msg *, size_t);
-NNG_DECL int    nng_msg_header_append(nng_msg *, const void *, size_t);
-NNG_DECL int    nng_msg_header_insert(nng_msg *, const void *, size_t);
-NNG_DECL int    nng_msg_header_trim(nng_msg *, size_t);
-NNG_DECL int    nng_msg_header_chop(nng_msg *, size_t);
-NNG_DECL int    nng_msg_header_append_u16(nng_msg *, uint16_t);
-NNG_DECL int    nng_msg_header_append_u32(nng_msg *, uint32_t);
-NNG_DECL int    nng_msg_header_append_u64(nng_msg *, uint64_t);
-NNG_DECL int    nng_msg_header_insert_u16(nng_msg *, uint16_t);
-NNG_DECL int    nng_msg_header_insert_u32(nng_msg *, uint32_t);
-NNG_DECL int    nng_msg_header_insert_u64(nng_msg *, uint64_t);
-NNG_DECL int    nng_msg_header_chop_u16(nng_msg *, uint16_t *);
-NNG_DECL int    nng_msg_header_chop_u32(nng_msg *, uint32_t *);
-NNG_DECL int    nng_msg_header_chop_u64(nng_msg *, uint64_t *);
-NNG_DECL int    nng_msg_header_trim_u16(nng_msg *, uint16_t *);
-NNG_DECL int    nng_msg_header_trim_u32(nng_msg *, uint32_t *);
-NNG_DECL int    nng_msg_header_trim_u64(nng_msg *, uint64_t *);
-NNG_DECL int    nng_msg_append_u16(nng_msg *, uint16_t);
-NNG_DECL int    nng_msg_append_u32(nng_msg *, uint32_t);
-NNG_DECL int    nng_msg_append_u64(nng_msg *, uint64_t);
-NNG_DECL int    nng_msg_insert_u16(nng_msg *, uint16_t);
-NNG_DECL int    nng_msg_insert_u32(nng_msg *, uint32_t);
-NNG_DECL int    nng_msg_insert_u64(nng_msg *, uint64_t);
-NNG_DECL int    nng_msg_chop_u16(nng_msg *, uint16_t *);
-NNG_DECL int    nng_msg_chop_u32(nng_msg *, uint32_t *);
-NNG_DECL int    nng_msg_chop_u64(nng_msg *, uint64_t *);
-NNG_DECL int    nng_msg_trim_u16(nng_msg *, uint16_t *);
-NNG_DECL int    nng_msg_trim_u32(nng_msg *, uint32_t *);
-NNG_DECL int    nng_msg_trim_u64(nng_msg *, uint64_t *);
-NNG_DECL int    nng_msg_dup(nng_msg **, const nng_msg *);
-NNG_DECL void   nng_msg_clear(nng_msg *);
-NNG_DECL void   nng_msg_header_clear(nng_msg *);
-NNG_DECL void   nng_msg_set_pipe(nng_msg *, nng_pipe);
+NNG_DECL int      nng_msg_alloc(nng_msg **, size_t);
+NNG_DECL void     nng_msg_free(nng_msg *);
+NNG_DECL int      nng_msg_realloc(nng_msg *, size_t);
+NNG_DECL void *   nng_msg_header(nng_msg *);
+NNG_DECL size_t   nng_msg_header_len(const nng_msg *);
+NNG_DECL void *   nng_msg_body(nng_msg *);
+NNG_DECL size_t   nng_msg_len(const nng_msg *);
+NNG_DECL int      nng_msg_append(nng_msg *, const void *, size_t);
+NNG_DECL int      nng_msg_insert(nng_msg *, const void *, size_t);
+NNG_DECL int      nng_msg_trim(nng_msg *, size_t);
+NNG_DECL int      nng_msg_chop(nng_msg *, size_t);
+NNG_DECL int      nng_msg_header_append(nng_msg *, const void *, size_t);
+NNG_DECL int      nng_msg_header_insert(nng_msg *, const void *, size_t);
+NNG_DECL int      nng_msg_header_trim(nng_msg *, size_t);
+NNG_DECL int      nng_msg_header_chop(nng_msg *, size_t);
+NNG_DECL int      nng_msg_header_append_u16(nng_msg *, uint16_t);
+NNG_DECL int      nng_msg_header_append_u32(nng_msg *, uint32_t);
+NNG_DECL int      nng_msg_header_append_u64(nng_msg *, uint64_t);
+NNG_DECL int      nng_msg_header_insert_u16(nng_msg *, uint16_t);
+NNG_DECL int      nng_msg_header_insert_u32(nng_msg *, uint32_t);
+NNG_DECL int      nng_msg_header_insert_u64(nng_msg *, uint64_t);
+NNG_DECL int      nng_msg_header_chop_u16(nng_msg *, uint16_t *);
+NNG_DECL int      nng_msg_header_chop_u32(nng_msg *, uint32_t *);
+NNG_DECL int      nng_msg_header_chop_u64(nng_msg *, uint64_t *);
+NNG_DECL int      nng_msg_header_trim_u16(nng_msg *, uint16_t *);
+NNG_DECL int      nng_msg_header_trim_u32(nng_msg *, uint32_t *);
+NNG_DECL int      nng_msg_header_trim_u64(nng_msg *, uint64_t *);
+NNG_DECL int      nng_msg_append_u16(nng_msg *, uint16_t);
+NNG_DECL int      nng_msg_append_u32(nng_msg *, uint32_t);
+NNG_DECL int      nng_msg_append_u64(nng_msg *, uint64_t);
+NNG_DECL int      nng_msg_insert_u16(nng_msg *, uint16_t);
+NNG_DECL int      nng_msg_insert_u32(nng_msg *, uint32_t);
+NNG_DECL int      nng_msg_insert_u64(nng_msg *, uint64_t);
+NNG_DECL int      nng_msg_chop_u16(nng_msg *, uint16_t *);
+NNG_DECL int      nng_msg_chop_u32(nng_msg *, uint32_t *);
+NNG_DECL int      nng_msg_chop_u64(nng_msg *, uint64_t *);
+NNG_DECL int      nng_msg_trim_u16(nng_msg *, uint16_t *);
+NNG_DECL int      nng_msg_trim_u32(nng_msg *, uint32_t *);
+NNG_DECL int      nng_msg_trim_u64(nng_msg *, uint64_t *);
+NNG_DECL int      nng_msg_dup(nng_msg **, const nng_msg *);
+NNG_DECL void     nng_msg_clear(nng_msg *);
+NNG_DECL void     nng_msg_header_clear(nng_msg *);
+NNG_DECL void     nng_msg_set_pipe(nng_msg *, nng_pipe);
 NNG_DECL nng_pipe nng_msg_get_pipe(const nng_msg *);
-NNG_DECL int      nng_msg_getopt(nng_msg *, int, void *, size_t *);
+
+// nng_msg_getopt is defunct, and should not be used by programs. It
+// always returns NNG_ENOTSUP.
+NNG_DECL int nng_msg_getopt(nng_msg *, int, void *, size_t *);
 
 // Pipe API. Generally pipes are only "observable" to applications, but
 // we do permit an application to close a pipe. This can be useful, for
@@ -746,10 +756,10 @@ NNG_DECL int nng_pipe_get_string(nng_pipe, const char *, char **);
 NNG_DECL int nng_pipe_get_ptr(nng_pipe, const char *, void **);
 NNG_DECL int nng_pipe_get_addr(nng_pipe, const char *, nng_sockaddr *);
 
-NNG_DECL int nng_pipe_close(nng_pipe);
-NNG_DECL int nng_pipe_id(nng_pipe);
-NNG_DECL nng_socket nng_pipe_socket(nng_pipe);
-NNG_DECL nng_dialer nng_pipe_dialer(nng_pipe);
+NNG_DECL int          nng_pipe_close(nng_pipe);
+NNG_DECL int          nng_pipe_id(nng_pipe);
+NNG_DECL nng_socket   nng_pipe_socket(nng_pipe);
+NNG_DECL nng_dialer   nng_pipe_dialer(nng_pipe);
 NNG_DECL nng_listener nng_pipe_listener(nng_pipe);
 
 // Flags.
@@ -850,9 +860,9 @@ enum nng_flag_enum {
 // Local TCP port number.  This is used on a listener, and is intended
 // to be used after starting the listener in combination with a wildcard
 // (0) local port.  This determines the actual ephemeral port that was
-// selected and bound.  The value is provied as an int, but only the
+// selected and bound.  The value is provided as an int, but only the
 // low order 16 bits will be set.  This is provided in native byte order,
-// which makes it more convienent than using the NNG_OPT_LOCADDR option.
+// which makes it more convenient than using the NNG_OPT_LOCADDR option.
 #define NNG_OPT_TCP_BOUND_PORT "tcp-bound-port"
 
 // IPC options.  These will largely vary depending on the platform,
@@ -913,7 +923,7 @@ enum nng_flag_enum {
 // This can be useful when a handler supports an entire directory tree.
 #define NNG_OPT_WS_REQUEST_URI "ws:request-uri"
 
-// NNG_OPT_WS_TXFRAMESZ is used to configure the fragmentation size
+// NNG_OPT_WS_SENDMAXFRAME is used to configure the fragmentation size
 // used for frames.  This has a default value of 64k.  Large values
 // are good for throughput, but penalize latency.  They also require
 // additional buffering on the peer.  This value must not be larger
@@ -921,7 +931,7 @@ enum nng_flag_enum {
 // to negotiate this.
 #define NNG_OPT_WS_SENDMAXFRAME "ws:txframe-max"
 
-// NNG_OPT_WS_RXFRAMESZ is the largest frame we will accept.  This should
+// NNG_OPT_WS_RECVMAXFRAME is the largest frame we will accept.  This should
 // probably not be larger than NNG_OPT_RECVMAXSZ. If the sender attempts
 // to send more data than this in a single message, it will be dropped.
 #define NNG_OPT_WS_RECVMAXFRAME "ws:rxframe-max"
@@ -941,7 +951,7 @@ enum nng_flag_enum {
 // nng_stats_get takes a snapshot of the entire set of statistics.
 // While the operation can be somewhat expensive (allocations), it
 // is done in a way that minimizes impact to running operations.
-// Note that the statistics are provided as tree, with parents
+// Note that the statistics are provided as a tree, with parents
 // used for grouping, and with child statistics underneath.  The
 // top stat returned will be of type NNG_STAT_SCOPE with name "".
 // Applications may choose to consider this root scope as "root", if
@@ -971,13 +981,25 @@ NNG_DECL nng_stat *nng_stat_child(nng_stat *);
 NNG_DECL const char *nng_stat_name(nng_stat *);
 
 // nng_stat_type is used to determine the type of the statistic.
-// At present, only NNG_STAT_TYPE_LEVEL and and NNG_STAT_TYPE_COUNTER
-// are defined.  Counters generally increment, and therefore changes in the
-// value over time are likely more interesting than the actual level.  Level
+// Counters generally increment, and therefore changes in the value over
+// time are likely more interesting than the actual level.  Level
 // values reflect some absolute state however, and should be presented to the
 // user as is.
 NNG_DECL int nng_stat_type(nng_stat *);
 
+// nng_stat_find is used to find a specific named statistic within
+// a statistic tree.  NULL is returned if no such statistic exists.
+NNG_DECL nng_stat *nng_stat_find(nng_stat *, const char *);
+
+// nng_stat_find_socket is used to find the stats for the given socket.
+NNG_DECL nng_stat *nng_stat_find_socket(nng_stat *, nng_socket);
+
+// nng_stat_find_dialer is used to find the stats for the given dialer.
+NNG_DECL nng_stat *nng_stat_find_dialer(nng_stat *, nng_dialer);
+
+// nng_stat_find_listener is used to find the stats for the given listener.
+NNG_DECL nng_stat *nng_stat_find_listener(nng_stat *, nng_listener);
+
 enum nng_stat_type_enum {
 	NNG_STAT_SCOPE   = 0, // Stat is for scoping, and carries no value
 	NNG_STAT_LEVEL   = 1, // Numeric "absolute" value, diffs meaningless
@@ -1031,7 +1053,7 @@ NNG_DECL int nng_device(nng_socket, nng_socket);
 
 // Error codes.  These generally have different values from UNIX errnos,
 // so take care about converting them.  The one exception is that 0 is
-// unambigiously "success".
+// unambiguously "success".
 //
 // NNG_SYSERR is a special code, which allows us to wrap errors from the
 // underlying operating system.  We generally prefer to map errors to one
@@ -1241,4 +1263,4 @@ NNG_DECL int nng_stream_listener_set_addr(
 }
 #endif
 
-#endif // NNG_H
+#endif // NNG_NNG_H
