@@ -1,5 +1,5 @@
 //
-// Copyright 2019 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -43,9 +43,9 @@ static nni_aio *nni_aio_expire_aio;
 //
 // The aio framework is tightly bound up with the taskq framework. We
 // "prepare" the task for an aio when a caller marks an aio as starting
-// (with nni_aio_begin), and that marks the task as bus. Then, all we have
+// (with nni_aio_begin), and that marks the task as busy. Then, all we have
 // to do is wait for the task to complete (the busy flag to be cleared)
-// when we want to know if the aio's operation itself is complete.
+// when we want to know if the operation itself is complete.
 //
 // In order to guard against aio reuse during teardown, we set the a_stop
 // flag.  Any attempt to initialize for a new operation after that point
@@ -61,123 +61,75 @@ static nni_aio *nni_aio_expire_aio;
 // operations from starting, without waiting for any existing one to
 // complete, call nni_aio_close.
 
-// An nni_aio is an async I/O handle.
-struct nng_aio {
-	int          a_result;  // Result code (nng_errno)
-	size_t       a_count;   // Bytes transferred (I/O only)
-	nni_time     a_expire;  // Absolute timeout
-	nni_duration a_timeout; // Relative timeout
-
-	// These fields are private to the aio framework.
-	bool      a_stop;    // shutting down (no new operations)
-	bool      a_sleep;   // sleeping with no action
-	int       a_sleeprv; // result when sleep wakes
-	nni_task *a_task;
-
-	// Read/write operations.
-	nni_iov *a_iov;
-	unsigned a_niov;
-	nni_iov  a_iovinl[4]; // inline IOVs - when the IOV list is short
-	nni_iov *a_iovalloc;  // dynamically allocated IOVs
-	unsigned a_niovalloc; // number of allocated IOVs
-
-	// Message operations.
-	nni_msg *a_msg;
-
-	// User scratch data.  Consumers may store values here, which
-	// must be preserved by providers and the framework.
-	void *a_user_data[4];
-
-	// Operation inputs & outputs.  Up to 4 inputs and 4 outputs may be
-	// specified.  The semantics of these will vary, and depend on the
-	// specific operation.
-	void *a_inputs[4];
-	void *a_outputs[4];
-
-	// Provider-use fields.
-	nni_aio_cancelfn a_cancel_fn;
-	void *           a_cancel_arg;
-	void *           a_prov_data;
-	nni_list_node    a_prov_node;
-	void *           a_prov_extra[4]; // Extra data used by provider
-
-	// Socket address.  This turns out to be very useful, as we wind up
-	// needing socket addresses for numerous connection related routines.
-	// It would be cleaner to not have this and avoid burning the space,
-	// but having this hear dramatically simplifies lots of code.
-	nng_sockaddr a_sockaddr;
-
-	// Expire node.
-	nni_list_node a_expire_node;
-};
-
 static void nni_aio_expire_add(nni_aio *);
 
-int
-nni_aio_init(nni_aio **aiop, nni_cb cb, void *arg)
+void
+nni_aio_init(nni_aio *aio, nni_cb cb, void *arg)
 {
-	nni_aio *aio;
-	int      rv;
-
-	if ((aio = NNI_ALLOC_STRUCT(aio)) == NULL) {
-		return (NNG_ENOMEM);
-	}
-	if ((rv = nni_task_init(&aio->a_task, NULL, cb, arg)) != 0) {
-		NNI_FREE_STRUCT(aio);
-		return (rv);
-	}
-	aio->a_expire    = NNI_TIME_NEVER;
-	aio->a_timeout   = NNG_DURATION_INFINITE;
-	aio->a_iov       = aio->a_iovinl;
-	aio->a_niovalloc = 0;
-	*aiop            = aio;
-	return (0);
+	memset(aio, 0, sizeof(*aio));
+	nni_task_init(&aio->a_task, NULL, cb, arg);
+	aio->a_expire  = NNI_TIME_NEVER;
+	aio->a_timeout = NNG_DURATION_INFINITE;
 }
 
 void
 nni_aio_fini(nni_aio *aio)
+{
+	nni_aio_cancelfn fn;
+	void *           arg;
+
+	// TODO: This probably could just use nni_aio_stop.
+
+	// This is like aio_close, but we don't want to dispatch
+	// the task.  And unlike aio_stop, we don't want to wait
+	// for the task.  (Because we implicitly do task_fini.)
+	nni_mtx_lock(&nni_aio_lk);
+	fn                = aio->a_cancel_fn;
+	arg               = aio->a_cancel_arg;
+	aio->a_cancel_fn  = NULL;
+	aio->a_cancel_arg = NULL;
+	aio->a_stop       = true;
+	nni_mtx_unlock(&nni_aio_lk);
+
+	if (fn != NULL) {
+		fn(aio, arg, NNG_ECLOSED);
+	}
+
+	// Wait for the aio to be "done"; this ensures that we don't
+	// destroy an aio from a "normal" completion callback while
+	// the expiration thread is working.
+
+	nni_mtx_lock(&nni_aio_lk);
+	while (nni_aio_expire_aio == aio) {
+		// TODO: It should be possible to remove this check!
+		if (nni_thr_is_self(&nni_aio_expire_thr)) {
+			nni_aio_expire_aio = NULL;
+			break;
+		}
+		nni_cv_wait(&nni_aio_expire_cv);
+	}
+	nni_mtx_unlock(&nni_aio_lk);
+	nni_task_fini(&aio->a_task);
+}
+
+int
+nni_aio_alloc(nni_aio **aiop, nni_cb cb, void *arg)
+{
+	nni_aio *aio;
+
+	if ((aio = NNI_ALLOC_STRUCT(aio)) == NULL) {
+		return (NNG_ENOMEM);
+	}
+	nni_aio_init(aio, cb, arg);
+	*aiop          = aio;
+	return (0);
+}
+
+void
+nni_aio_free(nni_aio *aio)
 {
 	if (aio != NULL) {
-
-		nni_aio_cancelfn fn;
-		void *           arg;
-
-		// This is like aio_close, but we don't want to dispatch
-		// the task.  And unlike aio_stop, we don't want to wait
-		// for the task.  (Because we implicitly do task_fini.)
-		nni_mtx_lock(&nni_aio_lk);
-		fn                = aio->a_cancel_fn;
-		arg               = aio->a_cancel_arg;
-		aio->a_cancel_fn  = NULL;
-		aio->a_cancel_arg = NULL;
-		aio->a_stop       = true;
-		nni_mtx_unlock(&nni_aio_lk);
-
-		if (fn != NULL) {
-			fn(aio, arg, NNG_ECLOSED);
-		}
-
-		// Wait for the aio to be "done"; this ensures that we don't
-		// destroy an aio from a "normal" completion callback while
-		// the expiration thread is working.
-
-		nni_mtx_lock(&nni_aio_lk);
-		while (nni_aio_expire_aio == aio) {
-			if (nni_thr_is_self(&nni_aio_expire_thr)) {
-				nni_aio_expire_aio = NULL;
-				break;
-			}
-			nni_cv_wait(&nni_aio_expire_cv);
-		}
-		nni_mtx_unlock(&nni_aio_lk);
-
-		nni_task_fini(aio->a_task);
-
-		// At this point the AIO is done.
-		if (aio->a_niovalloc > 0) {
-			NNI_FREE_STRUCTS(aio->a_iovalloc, aio->a_niovalloc);
-		}
-
+		nni_aio_fini(aio);
 		NNI_FREE_STRUCT(aio);
 	}
 }
@@ -185,39 +137,27 @@ nni_aio_fini(nni_aio *aio)
 int
 nni_aio_set_iov(nni_aio *aio, unsigned niov, const nni_iov *iov)
 {
+
+	if (niov > NNI_NUM_ELEMENTS((aio->a_iov))) {
+		return (NNG_EINVAL);
+	}
+
 	// Sometimes we are resubmitting our own io vector, with
-	// just a smaller niov.
-	if (aio->a_iov != iov) {
-		if ((niov > NNI_NUM_ELEMENTS(aio->a_iovinl)) &&
-		    (niov > aio->a_niovalloc)) {
-			nni_iov *newiov = NNI_ALLOC_STRUCTS(newiov, niov);
-			if (newiov == NULL) {
-				return (NNG_ENOMEM);
-			}
-			if (aio->a_niovalloc > 0) {
-				NNI_FREE_STRUCTS(
-				    aio->a_iovalloc, aio->a_niovalloc);
-			}
-			aio->a_iov       = newiov;
-			aio->a_iovalloc  = newiov;
-			aio->a_niovalloc = niov;
+	// just a smaller count.  We copy them only if we are not.
+	if (iov != &aio->a_iov[0]) {
+		for (unsigned i = 0; i < niov; i++) {
+			aio->a_iov[i] = iov[i];
 		}
-		if (niov <= NNI_NUM_ELEMENTS(aio->a_iovinl)) {
-			aio->a_iov = aio->a_iovinl;
-		} else {
-			aio->a_iov = aio->a_iovalloc;
-		}
-		memcpy(aio->a_iov, iov, niov * sizeof(nni_iov));
 	}
 	aio->a_niov = niov;
 	return (0);
 }
 
-// nni_aio_stop cancels any oustanding operation, and waits for the
+// nni_aio_stop cancels any outstanding operation, and waits for the
 // callback to complete, if still running.  It also marks the AIO as
 // stopped, preventing further calls to nni_aio_begin from succeeding.
 // To correctly tear down an AIO, call stop, and make sure any other
-// calles are not also stopped, before calling nni_aio_fini to release
+// callers are not also stopped, before calling nni_aio_free to release
 // actual memory.
 void
 nni_aio_stop(nni_aio *aio)
@@ -347,7 +287,7 @@ nni_aio_count(nni_aio *aio)
 void
 nni_aio_wait(nni_aio *aio)
 {
-	nni_task_wait(aio->a_task);
+	nni_task_wait(&aio->a_task);
 }
 
 int
@@ -363,9 +303,10 @@ nni_aio_begin(nni_aio *aio)
 		aio->a_cancel_arg = NULL;
 		aio->a_expire     = NNI_TIME_NEVER;
 		aio->a_sleep      = false;
+		aio->a_expire_ok  = false;
 		nni_mtx_unlock(&nni_aio_lk);
 
-		nni_task_dispatch(aio->a_task);
+		nni_task_dispatch(&aio->a_task);
 		return (NNG_ECANCELED);
 	}
 	aio->a_result     = 0;
@@ -375,7 +316,7 @@ nni_aio_begin(nni_aio *aio)
 	for (unsigned i = 0; i < NNI_NUM_ELEMENTS(aio->a_outputs); i++) {
 		aio->a_outputs[i] = NULL;
 	}
-	nni_task_prep(aio->a_task);
+	nni_task_prep(&aio->a_task);
 	nni_mtx_unlock(&nni_aio_lk);
 	return (0);
 }
@@ -459,9 +400,9 @@ nni_aio_finish_impl(
 	nni_mtx_unlock(&nni_aio_lk);
 
 	if (synch) {
-		nni_task_exec(aio->a_task);
+		nni_task_exec(&aio->a_task);
 	} else {
-		nni_task_dispatch(aio->a_task);
+		nni_task_dispatch(&aio->a_task);
 	}
 }
 
@@ -503,13 +444,6 @@ nni_aio_list_append(nni_list *list, nni_aio *aio)
 	nni_list_append(list, aio);
 }
 
-void
-nni_aio_list_prepend(nni_list *list, nni_aio *aio)
-{
-	nni_aio_list_remove(aio);
-	nni_list_prepend(list, aio);
-}
-
 void
 nni_aio_list_remove(nni_aio *aio)
 {
@@ -546,11 +480,11 @@ nni_aio_expire_add(nni_aio *aio)
 }
 
 static void
-nni_aio_expire_loop(void *notused)
+nni_aio_expire_loop(void *unused)
 {
 	nni_list *aios = &nni_aio_expire_aios;
 
-	NNI_ARG_UNUSED(notused);
+	NNI_ARG_UNUSED(unused);
 
 	for (;;) {
 		nni_aio_cancelfn fn;
@@ -584,7 +518,7 @@ nni_aio_expire_loop(void *notused)
 		// This aio's time has come.  Expire it, canceling any
 		// outstanding I/O.
 		nni_list_remove(aios, aio);
-		rv = aio->a_sleep ? aio->a_sleeprv : NNG_ETIMEDOUT;
+		rv = aio->a_expire_ok ? 0 : NNG_ETIMEDOUT;
 
 		if ((fn = aio->a_cancel_fn) != NULL) {
 			void *arg         = aio->a_cancel_arg;
@@ -609,18 +543,6 @@ nni_aio_expire_loop(void *notused)
 	}
 }
 
-void *
-nni_aio_get_prov_data(nni_aio *aio)
-{
-	return (aio->a_prov_data);
-}
-
-void
-nni_aio_set_prov_data(nni_aio *aio, void *data)
-{
-	aio->a_prov_data = data;
-}
-
 void *
 nni_aio_get_prov_extra(nni_aio *aio, unsigned index)
 {
@@ -678,8 +600,10 @@ nni_aio_iov_advance(nni_aio *aio, size_t n)
 		}
 		resid -= aio->a_iov[0].iov_len;
 		n -= aio->a_iov[0].iov_len;
-		aio->a_iov = &aio->a_iov[1];
 		aio->a_niov--;
+		for (unsigned i = 0; i < aio->a_niov; i++) {
+			aio->a_iov[i] = aio->a_iov[i + 1];
+		}
 	}
 	return (resid); // we might not have used all of n for this iov
 }
@@ -709,8 +633,8 @@ nni_sleep_aio(nng_duration ms, nng_aio *aio)
 	if (nni_aio_begin(aio) != 0) {
 		return;
 	}
-	aio->a_sleeprv = 0;
-	aio->a_sleep   = true;
+	aio->a_expire_ok = true;
+	aio->a_sleep     = true;
 	switch (aio->a_timeout) {
 	case NNG_DURATION_DEFAULT:
 	case NNG_DURATION_INFINITE:
@@ -720,8 +644,8 @@ nni_sleep_aio(nng_duration ms, nng_aio *aio)
 		// If the timeout on the aio is shorter than our sleep time,
 		// then let it still wake up early, but with NNG_ETIMEDOUT.
 		if (ms > aio->a_timeout) {
-			aio->a_sleeprv = NNG_ETIMEDOUT;
-			ms             = aio->a_timeout;
+			aio->a_expire_ok = false;
+			ms               = aio->a_timeout;
 		}
 	}
 	aio->a_expire = nni_clock() + ms;
