@@ -1,5 +1,5 @@
 //
-// Copyright 2019 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 // Copyright 2018 Devolutions <info@devolutions.net>
 //
@@ -27,7 +27,21 @@ struct nni_tcp_conn {
 	nni_tcp_dialer *dialer;
 	nni_reap_item   reap;
 };
-extern int  nni_posix_tcp_init(nni_tcp_conn **, nni_posix_pfd *);
+
+struct nni_tcp_dialer {
+	nni_list                connq; // pending connections
+	bool                    closed;
+	bool                    nodelay;
+	bool                    keepalive;
+	struct sockaddr_storage src;
+	size_t                  srclen;
+	nni_mtx                 mtx;
+	nni_atomic_u64          ref;
+	nni_atomic_bool         fini;
+};
+
+extern int  nni_posix_tcp_alloc(nni_tcp_conn **, nni_tcp_dialer *);
+extern void nni_posix_tcp_init(nni_tcp_conn *, nni_posix_pfd *);
 extern void nni_posix_tcp_start(nni_tcp_conn *, int, int);
 extern void nni_posix_tcp_dialer_rele(nni_tcp_dialer *);
 
